==> 0.2.5.tar.gz: lib/fst.dart <==
import 'package:flutter/widgets.dart';
import 'package:state_tree_interface/state_tree_interface.dart' as i;

export 'package:state_tree_interface/state_tree_interface.dart'
    show
        Failure,
        Late,
        Async,
        LateAsync,
        R,
        SetState,
        Success,
        Result,
        Pending,
        Loading;

class _StateTreeLeaf extends i.StateTreeLeaf implements i.UpdateReason {
  _StateTreeLeaf(super.parent, super.dependency);

  @override
  StateTreeNode get parent => super.parent as StateTreeNode;
}

class _StateLeaf<T> extends _StateTreeLeaf {
  T value;
  _StateLeaf(StateTreeNode parent, this.value) : super(parent, const Object());

  void setState(T value) {
    if (value == this.value) return;
    this.value = value;
    parent.signalUpdate(this);
  }

  @override
  _StateLeaf update(_StateLeaf newLeaf) {
    value = newLeaf.value;

    super.update(newLeaf);
    return this;
  }
}

class _ProviderLeaf<T> extends _StateTreeLeaf {
  T value;
  Object? token;

  _ProviderLeaf(super.parent, super.dependency, this.value, this.token);

  @override
  void init() {
    super.init();

    if (parent._element._providerLeafs.containsKey((T, token))) {
      throw StateError(
        'Provider already exists for $T'
        '${token == null ? '.' : 'with token $token.'}',
      );
    }

    parent._element._providerLeafs[(T, token)] = this;
  }

  @override
  _ProviderLeaf<T> update(_ProviderLeaf<T> leaf) {
    dispose();
    leaf.init();

    super.update(leaf);
    return leaf;
  }

  @override
  void dispose() {
    parent._element._providerLeafs.remove((T, token));
    super.dispose();
  }
}

class _ConsumerLeaf<T, K> extends _StateTreeLeaf {
  final Object? token;
  final T Function(K value)? _query;

  late (Type, Object?) providerKey = (K, token);
  late T value;

  final bool subscribe;

  _ConsumerLeaf(
    StateTreeNode parent,
    this._query, {
    this.token,
    this.subscribe = true,
  }) : super(parent, const ());

  @override
  void init() {
    super.init();
    value = query(
      _Provider.of<T, K>(
        parent._element,
        aspect: this,
        token: token,
        subscribe: subscribe,
      ).value,
    );
  }

  T _identity(K value) => value as T;

  T query(K value) {
    return (_query ?? _identity)(value);
  }

  bool shouldUpdate(K oldValue, K newValue) {
    final a = query(oldValue);
    final b = query(newValue);

    final shouldUpdate = a != b;

    if (shouldUpdate) {
      value = b;
    }

    return shouldUpdate;
  }
}

class StateTreeNode implements i.StateTreeNode {
  final _StateTreeWidgetElement _element;

  const StateTreeNode(this._element);

  @override
  void signalUpdate(i.UpdateReason reason) {
    _element.markNeedsBuild();
  }

  @override
  (T, i.SetState<T>) state<T>(T initialState) {
    final leaf = _StateLeaf(this, initialState);
    final mountedLeaf = _element._next(leaf);

    return (mountedLeaf.value, mountedLeaf.setState);
  }

  @override
  void provide<T>(T value, {Object? dependency, Object? token}) {
    final leaf = _ProviderLeaf<T>(this, dependency ?? (value,), value, token);
    _element._next(leaf);
  }

  @override
  T consume<T>({Object? token}) {
    final leaf = _ConsumerLeaf<T, T>(this, null, token: token);
    return _element._next(leaf).value;
  }

  @override
  T read<T>({Object? token}) {
    return _Provider.of<T, T>(
      _element,
      token: token,
    ).value;
  }

  @override
  T query<T, K>(T Function(K value) query, {Object? token}) {
    final leaf = _ConsumerLeaf<T, K>(this, query, token: token);
    return _element._next(leaf).value;
  }

  @override
  void effect(void Function()? Function() effect, {Object? dependency}) {
    final leaf = _EffectLeaf(this, dependency ?? const Object(), effect);
    _element._next(leaf);
  }

  i.LateResult<T> stream<T>(Stream<T> stream, {Object? dependency}) {
    final (v, setValue) = state(i.Late<T>.pending());

    effect(
      () {
        final subscription = stream.listen(
          (v) => setValue(i.Late<T>.success(v)),
          onError: (err) => setValue(i.Late<T>.failure(err)),
        );

        return () => subscription.cancel();
      },
      dependency: dependency,
    );

    return v;
  }

  void action<T extends Intent, K>(
    K Function(BuildContext context, T intent) callback,
  ) {
    final leaf = _ActionLeaf<T, K>(this, callback);
    _element._next(leaf);
  }

  T create<T>(T Function() ctor, {Object? dependency}) {
    final leaf = _CreateLeaf(this, dependency ?? Object(), ctor);
    return _element._next(leaf).value;
  }
}

abstract class _StateTreeWidget extends Widget {
  const _StateTreeWidget({Key? key}) : super(key: key);

  Widget build(BuildContext context, StateTreeNode n);

  @override
  Element createElement() {
    return _StateTreeWidgetElement(this);
  }
}

class _StateTreeWidgetElement extends ComponentElement {
  late final node = StateTreeNode(this);
  int cursor = 0;
  final _leafs = <i.StateTreeLeaf>[];
  final _providerLeafs = <(Type, Object?), _ProviderLeaf>{};
  final _actionLeafs = <_ActionLeaf>[];

  _StateTreeWidgetElement(_StateTreeWidget widget) : super(widget);

  @override
  _StateTreeWidget get widget => super.widget as _StateTreeWidget;

  bool _isBuilding = false;

  T _next<T extends i.StateTreeLeaf>(T leaf) {
    if (!_isBuilding) {
      throw StateError(
        '$leaf could only be used synchronously during build.',
      );
    }

    if (cursor < _leafs.length) {
      final existing = _leafs[cursor];
      if (existing.dependency == leaf.dependency) {
        cursor++;
        return existing as T;
      } else {
        final newLeaf = existing.update(leaf);
        _leafs[cursor] = newLeaf;
        cursor++;
        return newLeaf as T;
      }
    }

    leaf.init();
    _leafs.add(leaf);
    cursor++;

    return leaf;
  }

  @override
  Widget build() {
    cursor = 0;

    _isBuilding = true;
    var child = widget.build(this, node);
    _isBuilding = false;

    for (int j = cursor; j < _leafs.length; j++) {
      final leaf = _leafs[j];
      leaf.dispose();
    }

    _leafs.removeRange(cursor, _leafs.length);

    for (var element in _leafs) {
      element.cleanup();
    }

    if (_providerLeafs.isNotEmpty) {
      child = _Provider(
        leafs: {..._providerLeafs},
        parentElement: this,
        child: child,
      );
    }

    if (_actionLeafs.isNotEmpty) {
      child = Actions(
        actions: {
          for (final leaf in _actionLeafs) leaf.intentType: leaf.action,
        },
        child: child,
      );
    }

    return child;
  }

  @override
  void unmount() {
    for (final leaf in _leafs) {
      leaf.dispose();
    }

    super.unmount();
  }
}

class _Provider extends InheritedModel<_ConsumerLeaf> {
  final Map<(Type, Object?), _ProviderLeaf> leafs;
  final Element parentElement;

  const _Provider({
    required super.child,
    required this.leafs,
    required this.parentElement,
  });

  @override
  bool updateShouldNotify(_Provider oldWidget) {
    final oldKeys = oldWidget.leafs.keys.toSet();
    final newKeys = leafs.keys.toSet();

    final shouldNotify = oldKeys.difference(newKeys).isNotEmpty;

    if (shouldNotify) return shouldNotify;

    for (final key in leafs.keys) {
      final oldLeaf = oldWidget.leafs[key];
      final newLeaf = leafs[key];

      if (oldLeaf!.value != newLeaf!.value) {
        return true;
      }
    }

    return false;
  }

  @override
  bool updateShouldNotifyDependent(
    _Provider oldWidget,
    Set<_ConsumerLeaf> dependencies,
  ) {
    final shouldNotify = dependencies.any((e) {
      final key = e.providerKey;

      final oldLeaf = oldWidget.leafs[key];
      final newLeaf = leafs[key];

      if (oldLeaf == null || newLeaf == null) return true;

      return e.shouldUpdate(oldLeaf.value, newLeaf.value);
    });

    return shouldNotify;
  }

  static _ProviderLeaf<K> of<T, K>(
    Element element, {
    _ConsumerLeaf<T, K>? aspect,
    bool subscribe = false,
    Object? token,
  }) {
    Element? e = element;

    while (true) {
      final ie = e?.getElementForInheritedWidgetOfExactType<_Provider>();
      final model = ie?.widget as _Provider?;

      if (model == null) {
        throw StateError(
          'No provider found for $K'
          '${token == null ? '.' : 'with token $token'}',
        );
      }

      final leaf = model.leafs[(K, token)];

      if (leaf == null) {
        e = model.parentElement;
        continue;
      }

      if (subscribe) element.dependOnInheritedElement(ie!, aspect: aspect);

      return leaf as _ProviderLeaf<K>;
    }
  }
}

class _EffectLeaf extends _StateTreeLeaf {
  final void Function()? Function() effect;
  void Function()? unsubscribe;

  _EffectLeaf(super.parent, super.dependency, this.effect);

  @override
  void init() {
    super.init();
    unsubscribe = effect();
  }

  @override
  _EffectLeaf update(_EffectLeaf leaf) {
    dispose();
    leaf.init();

    super.update(leaf);
    return leaf;
  }

  @override
  void dispose() {
    unsubscribe?.call();
    super.dispose();
  }
}

class _Action<T extends Intent, K> extends ContextAction<T> {
  final _ActionLeaf<T, K> leaf;

  _Action(this.leaf);

  @override
  K invoke(T intent, [BuildContext? context]) {
    return leaf.callback(context!, intent);
  }
}

class _ActionLeaf<T extends Intent, K> extends _StateTreeLeaf {
  K Function(BuildContext context, T intent) callback;
  late final action = _Action(this);

  _ActionLeaf(StateTreeNode parent, this.callback) : super(parent, callback);

  Type get intentType => T;

  @override
  void init() {
    super.init();
    parent._element._actionLeafs.add(this);
  }

  @override
  i.StateTreeLeaf update(_ActionLeaf<T, K> newLeaf) {
    callback = newLeaf.callback;
    return super.update(newLeaf);
  }

  @override
  void dispose() {
    parent._element._actionLeafs.remove(this);
    super.dispose();
  }
}

class _CreateLeaf<T> extends _StateTreeLeaf {
  final T Function() ctor;
  late T value;

  _CreateLeaf(super.parent, super.dependency, this.ctor);

  @override
  void init() {
    super.init();
    value = ctor();
  }

  @override
  _CreateLeaf<T> update(_CreateLeaf<T> leaf) {
    value = leaf.ctor();
    super.update(leaf);

    return this;
  }
}

class StateTreeBuilder extends _StateTreeWidget {
  final Widget Function(
    BuildContext context,
    StateTreeNode n,
    Widget child,
  ) builder;

  final Widget child;

  const StateTreeBuilder(
    this.builder, {
    Key? key,
    this.child = const SizedBox(),
  }) : super(key: key);

  @override
  Widget build(BuildContext context, StateTreeNode n) {
    return builder(context, n, child);
  }
}

extension Read on BuildContext {
  T read<T>({Object? token}) {
    return _Provider.of<T, T>(
      this as Element,
      token: token,
    ).value;
  }
}

==> 0.2.6.tar.gz: lib/fst.dart <==
import 'package:flutter/widgets.dart';
import 'package:state_tree_interface/state_tree_interface.dart' as i;

export 'package:state_tree_interface/state_tree_interface.dart'
    show
        Failure,
        Late,
        Async,
        LateAsync,
        R,
        SetState,
        Success,
        Result,
        Pending,
        Loading,
        Kind;

class _StateTreeLeaf extends i.StateTreeLeaf implements i.UpdateReason {
  _StateTreeLeaf(super.parent, super.dependency);

  @override
  StateTreeNode get parent => super.parent as StateTreeNode;
}

class _StateLeaf<T> extends _StateTreeLeaf {
  T value;
  _StateLeaf(StateTreeNode parent, this.value) : super(parent, const Object());

  void setState(T value) {
    if (value == this.value) return;
    this.value = value;
    parent.signalUpdate(this);
  }
}

class _ProviderLeaf<T> extends _StateTreeLeaf {
  T value;
  i.Kind<T> kind;

  _ProviderLeaf(super.parent, super.dependency, this.value, this.kind);

  @override
  void init() {
    super.init();

    if (parent._element._providerLeafs.containsKey(kind)) {
      throw StateError(
        'Provider already exists for $T '
        '${'of kind $kind.'}',
      );
    }

    parent._element._providerLeafs[kind] = this;
  }

  @override
  _ProviderLeaf<T> update(_ProviderLeaf<T> leaf) {
    dispose();
    leaf.init();

    super.update(leaf);
    return leaf;
  }

  @override
  void dispose() {
    parent._element._providerLeafs.remove(kind);
    super.dispose();
  }
}

class _ConsumerLeaf<T, K> extends _StateTreeLeaf {
  final i.Kind<K> kind;
  final T Function(K value)? _query;
  late T value;

  final bool subscribe;

  _ConsumerLeaf(
    StateTreeNode parent,
    this._query, {
    required this.kind,
    this.subscribe = true,
  }) : super(parent, const ());

  @override
  void init() {
    super.init();
    value = query(
      _Provider.of<T, K>(
        parent._element,
        aspect: this,
        kind: kind,
        subscribe: subscribe,
      ).value,
    );
  }

  T _identity(K value) => value as T;

  T query(K value) {
    return (_query ?? _identity)(value);
  }

  bool shouldUpdate(K oldValue, K newValue) {
    final a = query(oldValue);
    final b = query(newValue);

    final shouldUpdate = a != b;

    if (shouldUpdate) {
      value = b;
    }

    return shouldUpdate;
  }
}

class StateTreeNode implements i.StateTreeNode {
  final _StateTreeWidgetElement _element;

  StateTreeNode(this._element);

  @override
  void signalUpdate(i.UpdateReason reason) {
    _element.markNeedsBuild();
  }

  @override
  (T, i.SetState<T>) state<T>(T initialState) {
    final leaf = _StateLeaf(this, initialState);
    final mountedLeaf = _element._next(leaf);

    return (mountedLeaf.value, mountedLeaf.setState);
  }

  @override
  void provide<T>(i.Kind<T> kind, T value, {Object? dependency}) {
    final leaf = _ProviderLeaf<T>(this, dependency ?? (value,), value, kind);
    _element._next(leaf);
  }

  @override
  T consume<T>(i.Kind<T> kind) {
    final leaf = _ConsumerLeaf<T, T>(this, null, kind: kind);
    return _element._next(leaf).value;
  }

  @override
  T read<T>(i.Kind<T> kind) {
    return _Provider.of<T, T>(
      _element,
      kind: kind,
    ).value;
  }

  @override
  T query<T, K>(i.Kind<K> kind, T Function(K value) query) {
    final leaf = _ConsumerLeaf<T, K>(this, query, kind: kind);
    return _element._next(leaf).value;
  }

  @override
  void effect(void Function()? Function() effect, {Object? dependency}) {
    final leaf = _EffectLeaf(this, dependency ?? const Object(), effect);
    _element._next(leaf);
  }

  i.LateResult<T> stream<T>(Stream<T> stream, {Object? dependency}) {
    final (v, setValue) = state(i.Late<T>.pending());

    effect(
      () {
        final subscription = stream.listen(
          (v) => setValue(i.Late<T>.success(v)),
          onError: (err) => setValue(i.Late<T>.failure(err)),
        );

        return () => subscription.cancel();
      },
      dependency: dependency,
    );

    return v;
  }

  late i.ActionRegistry<Intent> action = ActionRegistry(this);

  T create<T>(T Function() ctor, {Object? dependency}) {
    final leaf = _CreateLeaf(this, dependency ?? const Object(), ctor);
    return _element._next(leaf).value;
  }

  T map<T, K>(K value, T Function(K value) map) {
    final leaf = _MapLeaf<T, K>(this, value as Object, map);
    return _element._next(leaf).value;
  }
}

class ActionRegistry implements i.ActionRegistry<Intent> {
  final StateTreeNode owner;

  ActionRegistry(this.owner);

  @override
  void call<T extends Intent, K>(K Function(T intent) action) {
    final leaf = _ActionLeaf<T, K>(owner, action);
    owner._element._next(leaf);
  }

  @override
  void setValue<T>(void Function(T value) setValue) {
    call<SetValue<T>, void>((intent) {
      setValue(intent.value);
    });
  }
}

abstract class _StateTreeWidget extends Widget {
  const _StateTreeWidget({Key? key}) : super(key: key);

  Widget build(BuildContext context, StateTreeNode n);

  @override
  Element createElement() {
    return _StateTreeWidgetElement(this);
  }
}

class _StateTreeWidgetElement extends ComponentElement {
  late final node = StateTreeNode(this);
  int cursor = 0;
  final _leafs = <i.StateTreeLeaf>[];
  final _providerLeafs = <i.Kind, _ProviderLeaf>{};
  final _actionLeafs = <_ActionLeaf>[];

  _StateTreeWidgetElement(_StateTreeWidget widget) : super(widget);

  @override
  _StateTreeWidget get widget => super.widget as _StateTreeWidget;

  bool _isBuilding = false;

  T _next<T extends i.StateTreeLeaf>(T leaf) {
    if (!_isBuilding) {
      throw StateError(
        '$leaf could only be used synchronously during build.',
      );
    }

    if (cursor < _leafs.length) {
      final existing = _leafs[cursor];
      if (existing.dependency == leaf.dependency) {
        cursor++;
        return existing as T;
      } else {
        final newLeaf = existing.update(leaf);
        _leafs[cursor] = newLeaf;
        cursor++;
        return newLeaf as T;
      }
    }

    leaf.init();
    _leafs.add(leaf);
    cursor++;

    return leaf;
  }

  @override
  Widget build() {
    cursor = 0;

    _isBuilding = true;
    var child = widget.build(this, node);
    _isBuilding = false;

    for (int j = cursor; j < _leafs.length; j++) {
      final leaf = _leafs[j];
      leaf.dispose();
    }

    _leafs.removeRange(cursor, _leafs.length);

    for (var element in _leafs) {
      element.cleanup();
    }

    if (_providerLeafs.isNotEmpty) {
      child = _Provider(
        leafs: {..._providerLeafs},
        parentElement: this,
        child: child,
      );
    }

    if (_actionLeafs.isNotEmpty) {
      child = Actions(
        actions: {
          for (final leaf in _actionLeafs) leaf.intentType: leaf.action,
        },
        child: child,
      );
    }

    return child;
  }

  @override
  void unmount() {
    for (final leaf in _leafs) {
      leaf.dispose();
    }

    super.unmount();
  }
}

class _Provider extends InheritedModel<_ConsumerLeaf> {
  final Map<i.Kind, _ProviderLeaf> leafs;
  final Element parentElement;

  const _Provider({
    required super.child,
    required this.leafs,
    required this.parentElement,
  });

  @override
  bool updateShouldNotify(_Provider oldWidget) {
    final oldKeys = oldWidget.leafs.keys.toSet();
    final newKeys = leafs.keys.toSet();

    final shouldNotify = oldKeys.difference(newKeys).isNotEmpty;

    if (shouldNotify) return shouldNotify;

    for (final key in leafs.keys) {
      final oldLeaf = oldWidget.leafs[key];
      final newLeaf = leafs[key];

      if (oldLeaf!.value != newLeaf!.value) {
        return true;
      }
    }

    return false;
  }

  @override
  bool updateShouldNotifyDependent(
    _Provider oldWidget,
    Set<_ConsumerLeaf> dependencies,
  ) {
    final shouldNotify = dependencies.any((e) {
      final key = e.kind;

      final oldLeaf = oldWidget.leafs[key];
      final newLeaf = leafs[key];

      if (oldLeaf == null || newLeaf == null) return true;

      return e.shouldUpdate(oldLeaf.value, newLeaf.value);
    });

    return shouldNotify;
  }

  static _ProviderLeaf<K> of<T, K>(
    Element element, {
    _ConsumerLeaf<T, K>? aspect,
    bool subscribe = false,
    Object? kind,
  }) {
    Element? e = element;

    while (true) {
      final ie = e?.getElementForInheritedWidgetOfExactType<_Provider>();
      final model = ie?.widget as _Provider?;

      if (model == null) {
        throw StateError(
          'No provider found for $K'
          '${kind == null ? '.' : 'with kind $kind'}',
        );
      }

      final leaf = model.leafs[kind];

      if (leaf == null) {
        e = model.parentElement;
        continue;
      }

      if (subscribe) element.dependOnInheritedElement(ie!, aspect: aspect);

      return leaf as _ProviderLeaf<K>;
    }
  }
}

class _EffectLeaf extends _StateTreeLeaf {
  final void Function()? Function() effect;
  void Function()? unsubscribe;

  _EffectLeaf(super.parent, super.dependency, this.effect);

  @override
  void init() {
    super.init();
    unsubscribe = effect();
  }

  @override
  _EffectLeaf update(_EffectLeaf leaf) {
    dispose();
    leaf.init();

    super.update(leaf);
    return leaf;
  }

  @override
  void dispose() {
    unsubscribe?.call();
    super.dispose();
  }
}

class _Action<T extends Intent, K> extends Action<T> {
  final _ActionLeaf<T, K> leaf;

  _Action(this.leaf);

  @override
  K invoke(T intent) {
    return leaf.callback(intent);
  }
}

class _ActionLeaf<T extends Intent, K> extends _StateTreeLeaf {
  K Function(T intent) callback;
  late final action = _Action(this);

  _ActionLeaf(StateTreeNode parent, this.callback) : super(parent, callback);

  Type get intentType => T;

  @override
  void init() {
    super.init();
    parent._element._actionLeafs.add(this);
  }

  @override
  i.StateTreeLeaf update(_ActionLeaf<T, K> newLeaf) {
    callback = newLeaf.callback;
    return super.update(newLeaf);
  }

  @override
  void dispose() {
    parent._element._actionLeafs.remove(this);
    super.dispose();
  }
}

class _CreateLeaf<T> extends _StateTreeLeaf {
  final T Function() ctor;
  late T value;

  _CreateLeaf(super.parent, super.dependency, this.ctor);

  @override
  void init() {
    super.init();
    value = ctor();
  }

  @override
  _CreateLeaf<T> update(_CreateLeaf<T> leaf) {
    value = leaf.ctor();
    super.update(leaf);

    return this;
  }
}

class _MapLeaf<T, K> extends _StateTreeLeaf {
  late T value;
  final T Function(K value) map;

  _MapLeaf(super.parent, super.dependency, this.map);

  @override
  void init() {
    super.init();
    value = map(dependency as K);
  }

  @override
  _MapLeaf<T, K> update(_MapLeaf<T, K> leaf) {
    super.update(leaf);
    leaf.init();

    return leaf;
  }
}

class StateTreeBuilder extends _StateTreeWidget {
  final Widget Function(
    BuildContext context,
    StateTreeNode n,
    Widget child,
  ) builder;

  final Widget child;

  const StateTreeBuilder(
    this.builder, {
    Key? key,
    this.child = const SizedBox(),
  }) : super(key: key);

  @override
  Widget build(BuildContext context, StateTreeNode n) {
    return builder(context, n, child);
  }
}

class SetValue<T> extends Intent {
  final T value;
  const SetValue(this.value);
}
